@@ -1,4 +1,4 @@
-use super::{Completion, Error, Result};
+use super::{Error, Result};
 use core::{
     convert::Infallible,
     ops::{ControlFlow, FromResidual, Try},
@@ -130,8 +130,8 @@ impl Status {
     #[inline]
     #[allow(clippy::result_unit_err)]
     pub fn into_with_val<T>(self, val: impl FnOnce() -> T) -> Result<T, ()> {
-        if !self.is_error() {
-            Ok(Completion::new(self, val()))
+        if self.is_success() {
+            Ok(val())
         } else {
             Err(self.into())
         }
@@ -143,8 +143,8 @@ impl Status {
         self,
         err: impl FnOnce(Status) -> ErrData,
     ) -> Result<(), ErrData> {
-        if !self.is_error() {
-            Ok(self.into())
+        if self.is_success() {
+            Ok(())
         } else {
             Err(Error::new(self, err(self)))
         }
@@ -157,8 +157,8 @@ impl Status {
         val: impl FnOnce() -> T,
         err: impl FnOnce(Status) -> ErrData,
     ) -> Result<T, ErrData> {
-        if !self.is_error() {
-            Ok(Completion::new(self, val()))
+        if self.is_success() {
+            Ok(val())
         } else {
             Err(Error::new(self, err(self)))
         }
@@ -176,18 +176,18 @@ impl From<Status> for Result<(), ()> {
 pub struct StatusResidual(NonZeroUsize);
 
 impl Try for Status {
-    type Output = Completion<()>;
+    type Output = ();
     type Residual = StatusResidual;
 
     fn branch(self) -> ControlFlow<Self::Residual, Self::Output> {
         match NonZeroUsize::new(self.0) {
             Some(r) => ControlFlow::Break(StatusResidual(r)),
-            None => ControlFlow::Continue(Completion::from(self)),
+            None => ControlFlow::Continue(()),
         }
     }
 
-    fn from_output(output: Self::Output) -> Self {
-        output.status()
+    fn from_output(_output: Self::Output) -> Self {
+        Self::SUCCESS
     }
 }
 
@@ -223,3 +223,51 @@ impl From<ucs2::Error> for Status {
         }
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn test_status_to_result() {
+        assert!(Result::from(Status::SUCCESS).is_ok());
+        assert!(Result::from(Status::WARN_DELETE_FAILURE).is_err());
+        assert!(Result::from(Status::BUFFER_TOO_SMALL).is_err());
+
+        assert_eq!(Status::SUCCESS.into_with_val(|| 123).unwrap(), 123);
+        assert!(Status::WARN_DELETE_FAILURE.into_with_val(|| 123).is_err());
+        assert!(Status::BUFFER_TOO_SMALL.into_with_val(|| 123).is_err());
+
+        assert!(Status::SUCCESS.into_with_err(|_| 123).is_ok());
+        assert_eq!(
+            *Status::WARN_DELETE_FAILURE
+                .into_with_err(|_| 123)
+                .unwrap_err()
+                .data(),
+            123
+        );
+        assert_eq!(
+            *Status::BUFFER_TOO_SMALL
+                .into_with_err(|_| 123)
+                .unwrap_err()
+                .data(),
+            123
+        );
+
+        assert_eq!(Status::SUCCESS.into_with(|| 123, |_| 456).unwrap(), 123);
+        assert_eq!(
+            *Status::WARN_DELETE_FAILURE
+                .into_with(|| 123, |_| 456)
+                .unwrap_err()
+                .data(),
+            456
+        );
+        assert_eq!(
+            *Status::BUFFER_TOO_SMALL
+                .into_with(|| 123, |_| 456)
+                .unwrap_err()
+                .data(),
+            456
+        );
+    }
+}
