@@ -119,8 +119,17 @@ pub struct BootServices {
     ) -> Status,
 
     // Driver support services
-    connect_controller: usize,
-    disconnect_controller: usize,
+    connect_controller: unsafe extern "efiapi" fn(
+        controller: Handle,
+        driver_image: Option<Handle>,
+        remaining_device_path: *const DevicePath,
+        recursive: bool,
+    ) -> Status,
+    disconnect_controller: unsafe extern "efiapi" fn(
+        controller: Handle,
+        driver_image: Option<Handle>,
+        child: Option<Handle>,
+    ) -> Status,
 
     // Protocol open / close services
     open_protocol: extern "efiapi" fn(
@@ -145,7 +154,13 @@ pub struct BootServices {
         protocol_buffer: *mut *mut *const Guid,
         protocol_buffer_count: *mut usize,
     ) -> Status,
-    locate_handle_buffer: usize,
+    locate_handle_buffer: unsafe extern "efiapi" fn(
+        search_ty: i32,
+        proto: *const Guid,
+        key: *const c_void,
+        no_handles: &mut usize,
+        buf: &mut *mut Handle,
+    ) -> Status,
     locate_protocol: extern "efiapi" fn(
         proto: &Guid,
         registration: *mut c_void,
@@ -486,8 +501,8 @@ impl BootServices {
     pub fn check_event(&self, event: Event) -> Result<bool> {
         let status = unsafe { (self.check_event)(event) };
         match status {
-            Status::SUCCESS => Ok(true.into()),
-            Status::NOT_READY => Ok(false.into()),
+            Status::SUCCESS => Ok(true),
+            Status::NOT_READY => Ok(false),
             _ => Err(status.into()),
         }
     }
@@ -547,7 +562,7 @@ impl BootServices {
         let buffer_len = buffer_size / handle_size;
 
         match (buffer, status) {
-            (NULL_BUFFER, Status::BUFFER_TOO_SMALL) => Ok(buffer_len.into()),
+            (NULL_BUFFER, Status::BUFFER_TOO_SMALL) => Ok(buffer_len),
             (_, other_status) => other_status.into_with_val(|| buffer_len),
         }
     }
@@ -570,22 +585,60 @@ impl BootServices {
         }
     }
 
-    /// Load an EFI image from a buffer.
-    pub fn load_image_from_buffer(
+    /// Load an EFI image into memory and return a [`Handle`] to the image.
+    ///
+    /// There are two ways to load the image: by copying raw image data
+    /// from a source buffer, or by loading the image via the
+    /// [`SimpleFileSystem`] protocol. See [`LoadImageSource`] for more
+    /// details of the `source` parameter.
+    ///
+    /// The `parent_image_handle` is used to initialize the
+    /// `parent_handle` field of the [`LoadedImage`] protocol for the
+    /// image.
+    ///
+    /// If the image is successfully loaded, a [`Handle`] supporting the
+    /// [`LoadedImage`] and `LoadedImageDevicePath` protocols is
+    /// returned. The image can be started with [`start_image`] or
+    /// unloaded with [`unload_image`].
+    ///
+    /// [`start_image`]: BootServices::start_image
+    /// [`unload_image`]: BootServices::unload_image
+    pub fn load_image(
         &self,
         parent_image_handle: Handle,
-        source_buffer: &[u8],
-    ) -> Result<Handle> {
-        let boot_policy = 0;
-        let device_path = ptr::null();
-        let source_size = source_buffer.len();
+        source: LoadImageSource,
+    ) -> uefi::Result<Handle> {
+        let boot_policy;
+        let device_path;
+        let source_buffer;
+        let source_size;
+        match source {
+            LoadImageSource::FromBuffer { buffer, file_path } => {
+                // Boot policy is ignored when loading from source buffer.
+                boot_policy = 0;
+
+                device_path = file_path.map(|p| p as _).unwrap_or(ptr::null());
+                source_buffer = buffer.as_ptr();
+                source_size = buffer.len();
+            }
+            LoadImageSource::FromFilePath {
+                file_path,
+                from_boot_manager,
+            } => {
+                boot_policy = u8::from(from_boot_manager);
+                device_path = file_path;
+                source_buffer = ptr::null();
+                source_size = 0;
+            }
+        };
+
         let mut image_handle = MaybeUninit::uninit();
         unsafe {
             (self.load_image)(
                 boot_policy,
                 parent_image_handle,
                 device_path,
-                source_buffer.as_ptr(),
+                source_buffer,
                 source_size,
                 &mut image_handle,
             )
@@ -694,6 +747,44 @@ impl BootServices {
         unsafe { (self.set_watchdog_timer)(timeout, watchdog_code, data_len, data) }.into()
     }
 
+    /// Connect one or more drivers to a controller.
+    ///
+    /// Usually one disconnects and then reconnects certain drivers
+    /// to make them rescan some state that changed, e.g. reconnecting
+    /// a `BlockIO` handle after your app changed the partitions somehow.
+    pub fn connect_controller(
+        &self,
+        controller: Handle,
+        driver_image: Option<Handle>,
+        remaining_device_path: Option<&DevicePath>,
+        recursive: bool,
+    ) -> Result {
+        unsafe {
+            (self.connect_controller)(
+                controller,
+                driver_image,
+                remaining_device_path
+                    .map(|dp| dp as _)
+                    .unwrap_or(ptr::null()),
+                recursive,
+            )
+        }
+        .into_with_err(|_| ())
+    }
+
+    /// Disconnect one or more drivers from a controller.
+    ///
+    /// See [`connect_controller`][Self::connect_controller].
+    pub fn disconnect_controller(
+        &self,
+        controller: Handle,
+        driver_image: Option<Handle>,
+        child: Option<Handle>,
+    ) -> Result {
+        unsafe { (self.disconnect_controller)(controller, driver_image, child) }
+            .into_with_err(|_| ())
+    }
+
     /// Open a protocol interface for a handle.
     ///
     /// This function attempts to get the protocol implementation of a
@@ -783,6 +874,26 @@ impl BootServices {
         })
     }
 
+    /// Returns an array of handles that support the requested protocol in a buffer allocated from
+    /// pool.
+    pub fn locate_handle_buffer(&self, search_ty: SearchType) -> Result<HandleBuffer> {
+        let mut num_handles: usize = 0;
+        let mut buffer: *mut Handle = ptr::null_mut();
+
+        // Obtain the needed data from the parameters.
+        let (ty, guid, key) = match search_ty {
+            SearchType::AllHandles => (0, ptr::null(), ptr::null_mut()),
+            SearchType::ByProtocol(guid) => (2, guid as *const _, ptr::null_mut()),
+        };
+
+        unsafe { (self.locate_handle_buffer)(ty, guid, key, &mut num_handles, &mut buffer) }
+            .into_with_val(|| HandleBuffer {
+                boot_services: self,
+                count: num_handles,
+                buffer,
+            })
+    }
+
     /// Returns a protocol implementation, if present on the system.
     ///
     /// The caveats of `BootServices::handle_protocol()` also apply here.
@@ -823,14 +934,14 @@ impl BootServices {
         let search_type = SearchType::from_proto::<P>();
 
         // Determine how much we need to allocate.
-        let (status1, buffer_size) = self.locate_handle(search_type, None)?.split();
+        let buffer_size = self.locate_handle(search_type, None)?;
 
         // Allocate a large enough buffer without pointless initialization.
         let mut handles = Vec::with_capacity(buffer_size);
         let buffer = handles.spare_capacity_mut();
 
         // Perform the search.
-        let (status2, buffer_size) = self.locate_handle(search_type, Some(buffer))?.split();
+        let buffer_size = self.locate_handle(search_type, Some(buffer))?;
 
         // Mark the returned number of elements as initialized.
         unsafe {
@@ -838,9 +949,7 @@ impl BootServices {
         }
 
         // Emit output, with warnings
-        status1
-            .into_with_val(|| handles)
-            .map(|completion| completion.with_status(status2))
+        Ok(handles)
     }
 
     /// Retrieves the `SimpleFileSystem` protocol associated with
@@ -852,35 +961,29 @@ impl BootServices {
         &self,
         image_handle: Handle,
     ) -> Result<ScopedProtocol<SimpleFileSystem>> {
-        let loaded_image = self
-            .open_protocol::<LoadedImage>(
-                OpenProtocolParams {
-                    handle: image_handle,
-                    agent: image_handle,
-                    controller: None,
-                },
-                OpenProtocolAttributes::Exclusive,
-            )?
-            .expect("Failed to retrieve `LoadedImage` protocol from handle");
+        let loaded_image = self.open_protocol::<LoadedImage>(
+            OpenProtocolParams {
+                handle: image_handle,
+                agent: image_handle,
+                controller: None,
+            },
+            OpenProtocolAttributes::Exclusive,
+        )?;
         let loaded_image = unsafe { &*loaded_image.interface.get() };
 
         let device_handle = loaded_image.device();
 
-        let device_path = self
-            .open_protocol::<DevicePath>(
-                OpenProtocolParams {
-                    handle: device_handle,
-                    agent: image_handle,
-                    controller: None,
-                },
-                OpenProtocolAttributes::Exclusive,
-            )?
-            .expect("Failed to retrieve `DevicePath` protocol from image's device handle");
+        let device_path = self.open_protocol::<DevicePath>(
+            OpenProtocolParams {
+                handle: device_handle,
+                agent: image_handle,
+                controller: None,
+            },
+            OpenProtocolAttributes::Exclusive,
+        )?;
         let mut device_path = unsafe { &*device_path.interface.get() };
 
-        let device_handle = self
-            .locate_device_path::<SimpleFileSystem>(&mut device_path)?
-            .expect("Failed to locate `SimpleFileSystem` protocol on device path");
+        let device_handle = self.locate_device_path::<SimpleFileSystem>(&mut device_path)?;
 
         self.open_protocol::<SimpleFileSystem>(
             OpenProtocolParams {
@@ -1023,6 +1126,37 @@ impl Debug for BootServices {
     }
 }
 
+/// Used as a parameter of [`BootServices::load_image`] to provide the
+/// image source.
+pub enum LoadImageSource<'a> {
+    /// Load an image from a buffer. The data will copied from the
+    /// buffer, so the input reference doesn't need to remain valid
+    /// after the image is loaded.
+    FromBuffer {
+        /// Raw image data.
+        buffer: &'a [u8],
+
+        /// If set, this path will be added as the file path of the
+        /// loaded image. This is not required to load the image, but
+        /// may be used by the image itself to load other resources
+        /// relative to the image's path.
+        file_path: Option<&'a DevicePath>,
+    },
+
+    /// Load an image via the [`SimpleFileSystem`] protocol. If there is
+    /// no instance of that protocol associated with the path then the
+    /// behavior depends on `from_boot_manager`. If `true`, attempt to
+    /// load via the `LoadFile` protocol. If `false`, attempt to load
+    /// via the `LoadFile2` protocol, then fall back to `LoadFile`.
+    FromFilePath {
+        /// Device path from which to load the image.
+        file_path: &'a DevicePath,
+
+        /// Whether the request originates from the boot manager.
+        from_boot_manager: bool,
+    },
+}
+
 newtype_enum! {
 /// Task priority level.
 ///
@@ -1439,3 +1573,29 @@ impl<'a> ProtocolsPerHandle<'a> {
         unsafe { slice::from_raw_parts(self.protocols, self.count) }
     }
 }
+
+/// A buffer that contains an array of [`Handles`][Handle] that support the requested protocol.
+/// Returned by [`BootServices::locate_handle_buffer`].
+pub struct HandleBuffer<'a> {
+    // The pointer returned by `locate_handle_buffer` has to be free'd with
+    // `free_pool`, so keep a reference to boot services for that purpose.
+    boot_services: &'a BootServices,
+    count: usize,
+    buffer: *mut Handle,
+}
+
+impl<'a> Drop for HandleBuffer<'a> {
+    fn drop(&mut self) {
+        // Ignore the result, we can't do anything about an error here.
+        let _ = self.boot_services.free_pool(self.buffer as *mut u8);
+    }
+}
+
+impl<'a> HandleBuffer<'a> {
+    /// Get an array of [`Handles`][Handle] that support the requested protocol.
+    pub fn handles(&self) -> &[Handle] {
+        // convert raw pointer to slice here so that we can get
+        // appropriate lifetime of the slice.
+        unsafe { slice::from_raw_parts(self.buffer, self.count) }
+    }
+}
